#version 130

#include "/settings.glsl"

in vec2 uv;
uniform sampler2D colortex0;
uniform sampler2D depthtex0;

uniform float near, far;
uniform float blindness;
uniform float darknessFactor;
uniform vec3 fogColor;

uniform mat4 gbufferProjectionInverse;

vec3 projectAndDivide(mat4 projectionMatrix, vec3 position){
	vec4 homPos = projectionMatrix * vec4(position, 1.0);
	return homPos.xyz / homPos.w;
}

void main() {
	vec3 color = texture(colortex0, uv).rgb;
	
	#if FOG == 1
	float depth = texture(depthtex0, uv).r;
	if(depth == 1.0){
		/*DRAWBUFFERS:0*/
		gl_FragData[0] = vec4(color, 1.0f);
		return;
	}
	
	vec3 NDCPos = vec3(uv.xy, depth) * 2.0 - 1.0;
	vec3 viewPos = projectAndDivide(gbufferProjectionInverse, NDCPos);
	
	
	float distance = length(viewPos) / (far * mix(mix(1, near * 0.5, blindness), near, darknessFactor));
	float fogFactor = exp(-mix(FOG_DISTANCE, 1, blindness) * (1.0 - distance));
	
	color = mix(color, pow(fogColor, vec3(2.2f)), clamp(fogFactor, 0.0, 1.0));
	#endif
	
	/*DRAWBUFFERS:0*/
	gl_FragData[0] = vec4(color, 1.0f);
}
